searchData={"items":[{"type":"module","title":"ExUid2.Api.Sharing","doc":"This module provides the interface for the DSP to access the UID2 Operator Services's key sharing endpoint.","ref":"ExUid2.Api.Sharing.html"},{"type":"function","title":"ExUid2.Api.Sharing.fetch_keyring/0","doc":"Request the decryption keys necessary to decrypt UID2 tokens.","ref":"ExUid2.Api.Sharing.html#fetch_keyring/0"},{"type":"module","title":"ExUid2.Dsp","doc":"This is the client module for DSP-side decryption of UID2 tokens.","ref":"ExUid2.Dsp.html"},{"type":"function","title":"ExUid2.Dsp.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExUid2.Dsp.html#child_spec/1"},{"type":"function","title":"ExUid2.Dsp.decrypt_token/2","doc":"Attempts to decrypt a base64 encoded token.","ref":"ExUid2.Dsp.html#decrypt_token/2"},{"type":"function","title":"ExUid2.Dsp.refresh/0","doc":"","ref":"ExUid2.Dsp.html#refresh/0"},{"type":"function","title":"ExUid2.Dsp.start_link/1","doc":"","ref":"ExUid2.Dsp.html#start_link/1"},{"type":"function","title":"ExUid2.Dsp.start_refresh_loop/0","doc":"","ref":"ExUid2.Dsp.html#start_refresh_loop/0"},{"type":"module","title":"ExUid2.Keyring","doc":"Struct holding the keys periodically fetched from the UID2 operator server.\n\nFields:\n\n* `:keys` - The list of available keys\n\n* `:info` - Other information provided via the UID2 operator server's `/v2/keys/sharing` endpoint","ref":"ExUid2.Keyring.html"},{"type":"function","title":"ExUid2.Keyring.get_key/2","doc":"","ref":"ExUid2.Keyring.html#get_key/2"},{"type":"function","title":"ExUid2.Keyring.new/1","doc":"","ref":"ExUid2.Keyring.html#new/1"},{"type":"type","title":"ExUid2.Keyring.t/0","doc":"","ref":"ExUid2.Keyring.html#t:t/0"},{"type":"module","title":"ExUid2.Keyring.Key","doc":"Struct holding the information for a given key.\n\nFields:\n\n* `:activates_ms` - Time when the key becomes active (Unix timestamp in milliseconds)\n\n* `:created_ms` - Time when the key was created (Unix timestamp in milliseconds)\n\n* `:id` - The key's ID for lookups in the keyring\n\n* `:secret` - That key's secret that must be used to decrypt the UID2 tokens.\n\n* `keyset_id - Unknown","ref":"ExUid2.Keyring.Key.html"},{"type":"function","title":"ExUid2.Keyring.Key.new/1","doc":"","ref":"ExUid2.Keyring.Key.html#new/1"},{"type":"type","title":"ExUid2.Keyring.Key.t/0","doc":"","ref":"ExUid2.Keyring.Key.html#t:t/0"},{"type":"module","title":"ExUid2.Uid2","doc":"Struct holding the decrypted and parsed UID2 token\n\nFields:\n\n* `:uid` - User's unique ID\n\n* `:established_ms` - The time when the token was first created (Unix timestamp in milliseconds)\n\n* `:site_id` - The site's ID used for finding the right decryption key\n\n* `site_key` - The key found in the Keyring for the given `site_id`\n\n* `identity_scope` - The identity scope found in the keyring (should probably always be \"UID2\")\n\n* `identity_type` - Unknown\n\n* `advertising_token_version` - The token version.\n\n* `expires_ms` - The time after which the token will be expired (Unix timestamp in milliseconds)","ref":"ExUid2.Uid2.html"},{"type":"type","title":"ExUid2.Uid2.t/0","doc":"","ref":"ExUid2.Uid2.html#t:t/0"},{"type":"extras","title":"ExUid2","doc":"# ExUid2\n\nA library to interact with [Unified ID2](https://unifiedid.com/docs/intro)\n\nIt currently only handles the DSP part of UID2 (decrypting UID2 tokens in bid requests).","ref":"readme.html"},{"type":"extras","title":"Installation - ExUid2","doc":"Add :ex_uid2 to the list of dependencies in mix.exs\n\n```elixir\ndef deps do\n  [\n      {:ex_uid2, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - ExUid2","doc":"Add the ExUid2 configuration to your application's config file:\n\n```elixir\nimport Config\n\nconfig :ex_uid2,\n  base_url: \"https://uid2_operator_server.com\",\n  api_key: \"your_api_key\",\n  secret_key: \"your_secret_key\"\n\n```\n\nOnce started, the application will periodically request a fresh keyring from the configured UID2 operator server. Encrypted tokens\ncan then be decrypted.\n\n```elixir\nExUid2.Dsp.decrypt_token( )\n{:ok,\n %ExUid2.Uid2{\n   uid:  ,\n   established: ~U[2024-07-02 01:17:56.501Z],\n   site_id: 11,\n   site_key: %ExUid2.Keyring.Key{\n     activates: 1717269873,\n     created: 1717183473,\n     expires: 1725909873,\n     id: 2383,\n     secret:  ,\n     keyset_id: nil\n   },\n   identity_scope: \"UID2\",\n   identity_type: nil,\n   advertising_token_version: 2,\n   expires: ~U[2024-07-06 19:12:10.313Z]\n }}\n```\n\nIf the keyring hasn't been properly fetched by the time a token decryption is attempted, an error tuple will be returned instead:\n\n```elixir\nExUid2.Dsp.decrypt_token( )\n{:error, :no_keyring_stored}\n```","ref":"readme.html#usage"},{"type":"extras","title":"Token decryption specification - ExUid2","doc":"Since no specification for the token decryption protocol could be found, the decryption\nprotocol has been found by reverse engineering the existing SDKs and may include errors\nand misconceptions, so this specification may be erroneous or incomplete, but has been\nshown to work for decoding encrypted UID2 tokens.","ref":"readme.html#token-decryption-specification"},{"type":"extras","title":"General flow - ExUid2","doc":"1. A keyring is periocically updated by querying the `/v2/key/sharing` endpoint.\n2. The encrypted token envelope is fetched from the bid request\n3. The envelope is parsed to extract the token version, the master key ID, the master IV and the master payload\n4. The right key to decrypt the master payload is obtained by looking up the master key ID in the keyring.\n5. The master payload is decrypted using AES-256-CBC, the master key and the master IV.\n6. The decrypted master payload is parsed to extract the expiration timestamp, the site key ID, the identity IV and the identity payload.\n7. The expiration timestamp is checked to ensure the token is still valid.\n8. The right key to decrypt the identity is obtained by looking up the site key ID in the keyring.\n9. The indentity payload is decrypted using AES-256-CBC, the site key and the identity IV.\n10. The decrypted identity payload is parsed to extract the site ID, the identity length, the identity binary and the established timestamp.\n\n### `/v2/key/sharing` request\nWhile the `/v2/key/sharing` endpoint is undocumented, The workflow to query UID2's endpoints is documented, the \n[Encrypting Requests and Decrypting Responses](https://unifiedid.com/docs/getting-started/gs-encryption-decryption#encryption-and-decryption-code-examples) documentation seem to apply there. It is what \nthis library is doing.","ref":"readme.html#general-flow"},{"type":"extras","title":"Encrypted token envelope - ExUid2","doc":"| Offset (bytes) | Size (bytes) | Description |\n| -------------- | ------------ | ----------- |\n| 0              | 1            | UID2 token version |\n| 1              | 4            | Master key ID. Must be uint32 big endian. |\n| 5              | 16           | Master 128-bit initialization vector (IV), which is used to randomize data encryption. |\n| 21             | N            | Master Payload (encrypted) |","ref":"readme.html#encrypted-token-envelope"},{"type":"extras","title":"Decrypted Master Payload - ExUid2","doc":"| Offset (bytes) | Size (bytes) | Description |\n| -------------- | ------------ | ----------- |\n| 0              | 8            | Expiration time in milliseconds (Unix timestamp). Must be uint64 big endian. |\n| 8              | 4            | Site key ID. Must be uint32 big endian. |\n| 12             | 16           | Identity 128-bit initialization vector (IV), which is used to randomize data encryption. |\n| 28             | N            | Identity Payload (encrypted)","ref":"readme.html#decrypted-master-payload"},{"type":"extras","title":"Decrypted Identity - ExUid2","doc":"| Offset (bytes) | Size (bytes) | Description |\n| -------------- | ------------ | ----------- |\n| 0              | 4            | Site ID. Must be uint32 big endian. |\n| 4              | 4            | Length in bytes of ID binary. Must be uint32 big endian. |\n| 8              | N            | Base64-encoded ID binary (the actual user ID). |\n| 8 + N          | 4            | Unknown. |\n| 12 + N         | 8            | Time when the identity was established in milliseconds (Unix timestamp). Must be uint64 big endian. |\n| 20 + N         | M            | Unknown. |","ref":"readme.html#decrypted-identity"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}